'''
    用于文件下载
'''

import requests
import os
from urllib.request import urlopen, Request

import requests
from tqdm import tqdm

headers={'User-Agent':'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:60.0) Gecko/20100101 Firefox/60.0'}

def download_from_url(url, dst):
    """
    @param: url to download file
    @param: dst place to put the file
    :return: bool
    """
    # 获取文件长度
    try:
        file = Request(url,headers=headers)
        file_size = int(urlopen(file).info().get('Content-Length', -1))
    except Exception as e:
        print(e)
        print("错误，访问url: %s 异常" % url)
        return False

    # 判断本地文件存在时
    if os.path.exists(dst):
        # 获取文件大小
        first_byte = os.path.getsize(dst)
    else:
        # 初始大小为0
        first_byte = 0

    # 判断大小一致，表示本地文件存在
    if first_byte >= file_size:
        # print("文件已经存在,无需下载")
        return file_size

    # #删除不完整的文件
    # try:
    #     os.remove(dst)
    # except:
    #     pass
    
    header = {"Range": "bytes=%s-%s" % (first_byte, file_size)}
    pbar = tqdm(
        total=file_size, initial=first_byte,
        unit='B', unit_scale=True, desc=url.split('/')[-1])

    # 访问url进行下载
    req = requests.get(url, headers=headers, stream=True)

    #循环下载
    files = []
    for chunk in req.iter_content(chunk_size=1024):
        if chunk:
            files.append(chunk)
            pbar.update(1024)
        
    with(open(dst, 'ab')) as f:
        for chunk in files:
            f.write(chunk)

    pbar.close()
    return True


'''
    本类用于 情感倾向分析
    预测值为大小0～1之间的置信度
'''

import os
from sys import implementation

import numpy as np
import tensorflow as tf

from cemotion.dataset import DataSet
from cemotion.download import download_from_url


os.environ['TF_CPP_MIN_LOG_LEVEL'] = '2' #只显示error和warining信息


#检测所需文件是否存在，不存在则下载
def check_env(url, path):
    if os.path.exists(path):
        return
    else:
        print('Downloading the required environment, Please wait.')
        download_from_url(url, path)


class Cemotion:
    def __init__(self):
        current_path = os.path.dirname(__file__) #当前模块的路径
        #保存模型的路径
        model_path = current_path + '/models/rnn_emotion_1.0.h5'
        #保存中文词典路径
        dictionary_path = current_path + '/models/requirements/big_Chinese_Words_Map.dict'        
        #检测所需文件是否存在，判断是否下载
        check_env('https://www.cyberlight.xyz/static/file/cemotion/big_Chinese_Words_Map.dict', dictionary_path)
        check_env('https://www.cyberlight.xyz/static/file/cemotion/rnn_emotion_1.0.h5', model_path)
        #加载rnn模型
        self.__rnn = tf.keras.models.load_model(model_path)
        #加载数据集实例
        self.__dataset = DataSet(400, dictionary_path) #句子最大长度 #字典路径      
        
    def predict(self, text):
        #输入内容为文字时 返回 正负概率
        if type(text) == type('text mode'):
            # print('text mode')
            list_text = [text] #将文本转为列表
            #获取预测值  预测一个值时使用predict_on_batch
            prediction = self.__rnn.predict_on_batch(self.__dataset.data_to_train(list_text))[0][0]
            
            return round(prediction, 6)
        
        #输入列容为列表时 返回 带正负概率的列表
        elif type(text) == type(['list mode']) or type(text) == type(np.array(['list mode'])):
            #如果是numpy数组 则 转为列表
            if type(text) == type(np.array(['list mode'])):
                text = text.tolist()
            
            # print('list mode')
            list_text = text
            prediction = self.__rnn.predict(self.__dataset.data_to_train(list_text))
            
            list_new = [] #第一列保存文字，第二列保存文字对应的情感
            #生成新表
            for one, two in zip(list_text, prediction):
                list_new.append([one, round(two[0], 6)])
                
            return list_new


import setuptools


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="Cemotion-apple",
    version="0.2.6",
    author="Cyberbolt",
    author_email="735245473@qq.com",
    description="基于NLP的中文情感倾向分析库",
    long_description=long_description,
    long_description_content_type="text/markdown",    
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'tqdm>=4.56.0',
        'requests>=2.25.1',
        'jieba>=0.42.1',
    ]    
)


'''
此类用于数据转换
将 普通数据 转为模型能读懂的数据
'''

import jieba
import logging
import joblib
import numpy as np


jieba.setLogLevel(logging.INFO) #不输出jieba日志


class DataSet:
    def __init__(self, sentence_len, dictionary_path):
        #加载中文词典
        self.__Chinese_dict = joblib.load(dictionary_path) 
        self.sentence_len = int(sentence_len) #句子长度 单位:词
    
    #将每个句子的列表 转为 编码后的列表
    def data_to_train(self, list_str):
        Chinese_dict = self.__Chinese_dict #提取中文词典
        Chinese_reverse = {v:k for k,v in Chinese_dict.items()} #反转字典
        
        #将 句子表 编码
        list_coding = [] #保存编码的整个表
        #遍历每个句子
        for row in list_str:
            text_coding = [] #保存编码的句子
            
            #分词
            sentence = list(jieba.cut(row, cut_all=False) )
            #遍历句子中每个单词 #只保存词库有的单词
            for one in sentence:
                try:
                    text_coding.append(Chinese_reverse[one] ) #通过反转的字典查每个单词的索引
                except:
                    continue
            
            lenth = len(text_coding)
            #如果句子大于698个词，只取前698个，否则补齐698个
            if lenth > self.sentence_len:
                text_coding = text_coding[:self.sentence_len]
            elif lenth < self.sentence_len:
                text_coding.extend([0 for i in range(self.sentence_len-lenth)])
        
            list_coding.append(text_coding) #将编码后的句子保存到表
        
        return np.array(list_coding)